
//+------------------------------------------------------------------------
// CLASS Tree
//------------------------------------------------------------------------

function createTree() {
	var div;
	
	div = document.createElement('DIV');

	//+--------------- PROPERTIES
	div.subNodes = new Array();
	div.selectedNode = null;
	
	//+--------------- METHODS
	div.createNode = XTree_createNode;
	div.createNodeFrom = XTree_createNodeFrom;
	div.paint = XTree_paint;
	div.selectNode = XTree_selectNode;
	div.expandNode = XTree_expandNode;
	
	//+--------------- EVENTS
	makeEventDispatcher('XTree', div);
//	div.click = new Event(div, 'Click');
	div.fireSelect = XTree_fireSelect;
	
	return div;	
}


//------------------------------------------------------------------------
function XTree_paint() {
	var i;
	
	for (i = 0; i <= this.subNodes.length-1; i++) {
		if (this.subNodes[i].level == 0) {
			this.appendChild(this.subNodes[i]);
			this.subNodes[i].paint();
		}

	}
}

//------------------------------------------------------------------------
function XTree_selectNode(node) {
	if (typeof(node) == 'number') {
		node = this.subNodes[node];
	}

	if (this.selectedNode) {
		this.selectedNode.deselect();
	}
	
	node.select();	
	this.selectedNode = node;

	this.fireSelect();
}

//------------------------------------------------------------------------
function XTree_expandNode(node) {
	if (this.selectedNode) {
		this.selectedNode.deselect();
	}

	node.selected = true;
	node.expanded = node.expanded ? false : true;
	node.setIcons();
	node.setStyle();
	for (i = 0; i <= node.subNodes.length-1; i++) {
		node.subNodes[i].style.display = node.expanded ? '' : 'none';
	}

	this.selectedNode = node;
}

//+------------------------------------------------------------------------
// Creates a "Node" object which is the main componenet which you
// see on screen and interact with.
//------------------------------------------------------------------------
function XTree_createNode(text, action, index) {
	var div;
	
	div = document.createElement('DIV');
	this.subNodes[this.subNodes.length] = div;
	this.appendChild(div);
	
	// Properties
	div.tree = this;
	div.text = text;
	div.action = action;
	div.index = index;
	div.subNodes = new Array();
	div.isLeaf = false;
	div.level = 0;

	div.selected = false;
	div.expanded = true;
		
	// insert an image containing the switch icon
	div.switchImg = document.createElement('IMG');
	div.switchImg.width = 15;
	div.switchImg.height = 14;
	div.appendChild(div.switchImg);
	
	// insert an image containing the folder icon
	div.folderImg = document.createElement('IMG');
	div.folderImg.width = 16;
	div.folderImg.height = 14;
	div.folderImg.style.marginRight = 5;
	div.appendChild(div.folderImg);

	div.textSpan = document.createElement('SPAN');
	div.appendChild(div.textSpan);

	// Methods
	div.paint = XTreeNode_paint;
	div.addNode = XTreeNode_addNode;
	div.initStyle = XTreeNode_initStyle;
	div.setStyle = XTreeNode_setStyle;
	div.setIcons = XTreeNode_setIcons;
	div.select = XTreeNode_select;
	div.deselect = XTreeNode_deselect;
	div.expand = XTreeNode_expand;
	div.collapse = XTreeNode_collapse;
	
	div.attachEvent('onmouseup', XTreeNode_mouseUp);
	div.attachEvent('onmousedown', XTreeNode_mouseDown);
	div.attachEvent('ondblclick', XTreeNode_dblClick);
	div.attachEvent('onselectstart', XTreeNode_selectStart);
	
	div.style.display = 'none';
	
	return div;
}


//------------------------------------------------------------------------
function XTree_createNodeFrom(xmlNode) {
	var i, subs, node;
	
	node = this.createNode(	xmlNode.getAttribute("title"),
													xmlNode.getAttribute("name"),
													xmlNode.getAttribute("index"));
	subs = xmlNode.selectNodes('folder');
	for (i = 0; i <= subs.length-1; i++) {
		node.addNode(this.createNodeFrom(subs[i]));
	}
	
	return node;
}

function XTreeNode_paint() {
	var i;

	// set the style according to the type of node
	this.initStyle();
	this.setIcons();
	
	this.textSpan.innerHTML = this.text;
	
	this.innerHTML += "<br>";

	// indent according to level
	if (this.level > 0) {
		this.style.paddingLeft = 19;
	}

	for (i = 0; i <= this.subNodes.length-1; i++) {
		this.appendChild(this.subNodes[i]);
		this.subNodes[i].paint();
	}

	this.style.display = '';
}


//------------------------------------------------------------------------
function XTreeNode_addNode(node) {
	node.level = this.level+1;
	node.parent = this;
	this.subNodes[this.subNodes.length] = node;
}

//------------------------------------------------------------------------
function XTreeNode_initStyle() {
	this.textSpan.style.fontFamily = "Tahoma,Arial";
	this.textSpan.style.fontSize = "8pt";
	this.textSpan.style.backgroundColor = "#ffffff";
	
	this.style.cursor = "hand";
	this.style.backgroundImage = "url(XTreeDots.gif)";
	this.style.backgroundRepeat = "repeat-y";
	this.style.backgroundPosition = 3 + (this.level > 0 ? 20 : 0);
	this.style.marginTop = "1";
}

//------------------------------------------------------------------------
function XTreeNode_setStyle() {
	this.childNodes(2).style.backgroundColor = 
		this.selected ? '#208488' : '';
	this.childNodes(2).style.color = 
		this.selected ? '#ffffff' : '#000000';
}

//------------------------------------------------------------------------
function XTreeNode_setIcons() {
	if (this.subNodes.length == 0) {
		this.childNodes(0).src = 'XTreeBlank.gif';
	}
	else {
		this.childNodes(0).src = 
			this.expanded ? 'XTreeSwitchOn.gif' : 'XTreeSwitchOff.gif';
	}

	this.childNodes(1).src = 
		this.selected ? 'XTreeFolderOpen.gif' : 'XTreeFolderClosed.gif';
}

//------------------------------------------------------------------------
function XTreeNode_select() {
	this.selected = true;
	this.setStyle();
	this.setIcons();
}

//------------------------------------------------------------------------
function XTreeNode_deselect() {
	this.selected = false;
	this.setStyle();
	this.setIcons();
}

//------------------------------------------------------------------------
function XTreeNode_expand() {
	this.expanded = true;
	this.setStyle();
	this.setIcons();
}

//------------------------------------------------------------------------
function XTreeNode_collapse() {
	this.expanded = false;
	this.setStyle();
	this.setIcons();
}

//------------------------------------------------------------------------
function XTreeNode_mouseUp() {
}

//------------------------------------------------------------------------
function XTreeNode_mouseDown() {
	var node;

	node = event.srcElement.parentNode;
	node.tree.selectNode(node);

	event.cancelBubble = true;
}

//------------------------------------------------------------------------
function XTreeNode_dblClick() {
	var node;
	node = event.srcElement.parentNode;
	if (node.subNodes.length > 0) {
		node.tree.expandNode(node);
	}
		
	event.cancelBubble = true;
}

//------------------------------------------------------------------------
function XTreeNode_selectStart() {
	event.returnValue = false;
}

//------------------------------------------------------------------------
function XTree_fireSelect() {
	for (var i = 0; i <= this.listeners.length-1; i++) {
		if (this.listeners[i].onXTreeSelect) {
			this.listeners[i].onXTreeSelect(this);
		}
	}	
}